package com.chinacreator.datasync.services;

import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;
import java.util.List;
import java.util.concurrent.CountDownLatch;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.LinkedBlockingQueue;
import java.util.concurrent.ThreadPoolExecutor;
import java.util.concurrent.TimeUnit;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;

import com.chinacreator.datasync.common.CommonUtil;
import com.chinacreator.datasync.dao.FlagDealDao;
import com.chinacreator.datasync.entity.SyncLog;
import com.chinacreator.datasync.handler.SyncDealFail;
import com.chinacreator.datasync.handler.SyncDealSuccess;

@Component
public class SyncFlagService {
	
	public Logger logger = LoggerFactory.getLogger("flagdeal");
	
	@Value("${config.query.dealFail.pagesize}")
	public int sizeFail;
	
	@Value("${config.query.dealSuccess.pagesize}")
	public int sizeSuccess;
	
	@Autowired
    private FlagDealDao dao;
	
	private SimpleDateFormat f = new SimpleDateFormat("yyyy-MM-dd hh:mm:ss");
	
	/**
	 * 处理失败
	 * 
	 */
	public void dealFail(){
		//查询需要重新处理的数据
		List<SyncLog> list = dao.pageQueryDealFail(sizeFail);
		
		//多程处理需要同步的数据
		logger.info("DealFailJob  datasize ==> " + list.size());
		CountDownLatch threadSignal = new CountDownLatch(list.size());
		ExecutorService executorService = new ThreadPoolExecutor(5, 20, 10L, TimeUnit.MINUTES, new LinkedBlockingQueue<Runnable>(), new ThreadPoolExecutor.CallerRunsPolicy());
		for (SyncLog log : list) {
			SyncDealFail syncOrder = new SyncDealFail(log, list.size(), threadSignal, logger, dao);
			executorService.execute(syncOrder);
		}
		
		//等待多线程处理完
		executorService.shutdown();
		try {
			executorService.awaitTermination(1L, TimeUnit.MINUTES);
		} catch (InterruptedException e) {
			e.printStackTrace();
		}
		
		if (!CommonUtil.isNotBlank(list) || list.size()<(sizeFail/10)) {
			logger.info("DealFailJob 长时间未处理完的数据少 ............");
			CommonUtil.sleep(60000,logger);
			return;
		}
		
	}
	
	public void dealFail2(){
		//查询需要重新处理的数据
		List<SyncLog> list = dao.pageQueryDealFail2(sizeFail);
		
		//多程处理需要同步的数据
		logger.info("DealFailJob2  datasize ==> " + list.size());
		for (SyncLog log : list) {
			dao.updateOrderRelationFail(log.getId(), log.getUserid());
		}
	}
	
	/**
	 * 处理成功
	 * 
	 */
	public void dealSuccess(){
		//查询需要重新处理的数据
		List<SyncLog> list = dao.pageQueryDealSuccess(sizeSuccess);
		
		//多程处理需要同步的数据
		logger.info("DealSuccessJob  datasize ==> " + list.size()+"，"+f.format(new Date()));
		CountDownLatch threadSignal = new CountDownLatch(list.size());
		ExecutorService executorService = new ThreadPoolExecutor(20, 30, 10L, TimeUnit.MINUTES, new LinkedBlockingQueue<Runnable>(), new ThreadPoolExecutor.CallerRunsPolicy());
		for (SyncLog log : list) {
			SyncDealSuccess syncOrder = new SyncDealSuccess(log, list.size(), threadSignal, logger, dao);
			executorService.execute(syncOrder);
		}
		
		//等待多线程处理完
		executorService.shutdown();
		try {
			executorService.awaitTermination(1L, TimeUnit.MINUTES);
		} catch (InterruptedException e) {
			e.printStackTrace();
		}
		logger.info("DealSuccessJob处理完成，"+f.format(new Date()));
		if (!CommonUtil.isNotBlank(list) || list.size()<(sizeSuccess/10)) {
			logger.info("DealSuccessJob 长时间未处理完的数据少 ............");
			CommonUtil.sleep(60000,logger);
			return;
		}
	}
	
	/**
	 * 把两天前的数据移动到历史库
	 * 
	 */
	public void moveHis(){
		
		int inscount = 0;
		int delcount = 0;
		try {
			//查询两天前的时间点
			Calendar c = Calendar.getInstance();
			c.add(Calendar.DAY_OF_MONTH,-2);
			String dealtime = new SimpleDateFormat("yyyyMMddHHmmss").format(c.getTime());
			
			inscount =  dao.insertHisData(dealtime);
			if(inscount>0) {
				delcount = dao.deleteHisData(dealtime);
			}
			
		} catch (Exception e) {
			logger.info("MoveHisJob 异常:"+e.getClass().getName()+"|"+e.getMessage());
		}
		
		logger.info("MoveHisJob 迁移到历史表："+inscount+",清除操作表："+delcount);
	}
	
	
	
}
